/* Toolbar + graph */

.now {
  /*background: #ccc;*/
  overflow: hidden !important;
}

/* Toolbar */

.now .toolbar {
  margin: 5px;
}

.now .toolbar a.toollink,
.now .toolbar a.toollink:link,
.now .toolbar a.toollink:visited {
  color: black;
}

.now .toolbar a.toollink:active  {
  color: blue;
}

.now .toolbar {
  float: left;
  overflow: none;
  width: 252px;
  height: auto;
  z-index: 3;
  position: absolute;
}

.now .toolbar input[type=checkbox] {
    margin: 10px;
    display: none
}

.now .toolbar input[type=checkbox] + label {
    display:inline-block;
    margin:-2px;
    padding: 4px 12px;
    color: black;
    cursor: pointer;
}

.now .toolbar input[type=checkbox]:checked + label {
    color: blue;
}

/* Graph */

.now svg {
  background-color: #fff;
  cursor: default;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}

.now svg:not(.active):not(.ctrl) {
  cursor: crosshair;
}

/* Tooltip */

div.now-tooltip {
    position: absolute;
    left: 1px;
    right: 1px;
    max-height: 200px;
    max-width: 400px;
    overflow: auto;
    padding: 5px;
    padding-bottom: 10px;
    font: 12px sans-serif;
    background: #DDD;
    border: 0px;
    border-radius: 8px;
    z-index: 2000;
}

.now-tooltip.hidden {
    display: none;
}

.ipython-graph {
}

.ipython-graph:hover {
}

.hbefore {
  fill: red !important;
  font-size: 20px !important;
}

.hafter {
  fill: green !important;
  font-size: 20px !important;
}

.bottom .ipython-graph {
  display: inline-block;
}

.node-hover {
  stroke: blue !important;
}
